import planarity
from unittest import SkipTest

class TestPlanarityNetworkX:

    @classmethod
    def setup_class(self):
        try:
            global nx
            import networkx as nx
        except ImportError:
            raise SkipTest('networkx not available.')

        self.planar=[]
        self.planar.extend([nx.path_graph(5),
                            nx.complete_graph(4)])
        self.non_planar=[]
        self.non_planar.extend([nx.complete_graph(5),
                                nx.complete_bipartite_graph(3,3)])

    def test_is_planar(self):
        for G in self.planar:
            assert planarity.is_planar(G) is True
        for G in self.non_planar:
            assert planarity.is_planar(G)is False

    def test_is_planar_unions(self):
        try:
            from itertools import combinations,product
        except ImportError:
            raise SkipTest('itertools.combinations not found')

        for (G1,G2) in combinations(self.planar,2):
            G=nx.disjoint_union(G1,G2)
            assert planarity.is_planar(G) is True

        for (G1,G2) in combinations(self.non_planar,2):
            G=nx.disjoint_union(G1,G2)
            assert planarity.is_planar(G) is False

        for (G1,G2) in product(self.planar,self.non_planar):
            G=nx.disjoint_union(G1,G2)
            assert planarity.is_planar(G) is False

    def test_goldner_harary(self):
        # goldner-harary graph
        # http://en.wikipedia.org/wiki/Goldner%E2%80%93Harary_graph
        # a maximal planar graph
        e= [(1,2 ),( 1,3 ),( 1,4 ),( 1,5 ),( 1,7 ),( 1,8 ),( 1,10 ),
            ( 1,11 ),( 2,3 ),( 2,4 ),( 2,6 ),( 2,7 ),( 2,9 ),( 2,10 ),
            ( 2,11 ),( 3,4 ),( 4,5 ),( 4,6 ),( 4,7 ),( 5,7 ),( 6,7 ),
            ( 7,8 ),( 7,9 ),( 7,10 ),( 8,10 ),( 9,10 ),( 10,11)]
        G=nx.Graph(e)
        assert planarity.is_planar(G) is True

    def test_kuratowski_k5(self):
        G=nx.complete_graph(5)
        K=planarity.kuratowski_subgraph(G)
        assert frozenset(G.nodes()) == frozenset(K.nodes())
        assert frozenset(frozenset(x) for x in G.edges()) == frozenset(frozenset(x) for x in K.edges())

    def test_kuratowski_k5m(self):
        G=nx.complete_graph(5)
        G.remove_edge(1,2)
        K=planarity.kuratowski_subgraph(G)
        assert frozenset(K.nodes()) == frozenset()
        assert frozenset(K.edges()) == frozenset()

    def test_networkx_graph(self):
        G=nx.Graph()
        G.add_edge(100,200)
        G.add_edge(200,300)
        G.add_node(1000)
        P=planarity.PGraph(G)
        H=planarity.networkx_graph(P)
        assert frozenset(G.nodes()) == frozenset(H.nodes())
        assert frozenset(frozenset(x) for x in G.edges()) == frozenset(frozenset(x) for x in H.edges())



import planarity
# Example of the complete graph of 5 nodes, K5
# K5 is not planar

# use text strings as labels
edgelist = [('a', 'b'), ('a', 'c'), ('a', 'd'), ('a', 'e'),
            ('b', 'c'),('b', 'd'),('b', 'e'),
            ('c', 'd'), ('c', 'e'),
            ('d', 'e')]

print(planarity.is_planar(edgelist))  # False
# print forbidden Kuratowski subgraph (K5)
print(planarity.kuratowski_edges(edgelist))

# remove an edge
edgelist.remove(('a','b'))
# graph is now planar
print(planarity.is_planar(edgelist))  # True
# no forbidden subgraph, empty list returned
print(planarity.kuratowski_edges(edgelist))



import os
import tempfile

import planarity

class TestPlanarity:
    @classmethod
    def setup_class(self):
        self.k5_adj_symmetric = {0: {1: {}, 2: {}, 3: {}, 4: {}},
                     1: {0: {}, 2: {}, 3: {}, 4: {}},
                     2: {0: {}, 1: {}, 3: {}, 4: {}},
                     3: {0: {}, 1: {}, 2: {}, 4: {}},
                     4: {0: {}, 1: {}, 2: {}, 3: {}}}

        self.k5_adj = {0: {1: {}, 2: {}, 3: {}, 4: {}},
                     1: {2: {}, 3: {}, 4: {}},
                     2: {3: {}, 4: {}},
                     3: {4: {}},
                     4: {}}

        self.k5_adj_set = {0: set([1,2,3,4]),
                         1: set([2,3,4]),
                         2: set([3,4]),
                         3: set([4]),
                         4: set([])}

        self.k5_adj_list = {0: list([1,2,3,4]),
                         1: list([2,3,4]),
                         2: list([3,4]),
                         3: list([4]),
                         4: list([])}

        self.k5_edgelist = [(0, 1),
                          (0, 2),
                          (0, 3),
                          (0, 4),
                          (1, 2),
                          (1, 3),
                          (1, 4),
                          (2, 3),
                          (2, 4),
                          (3, 4)]

        self.p4_adj = {0: {1: {}},
                     1: {0: {}, 2: {}},
                     2: {1: {}, 3: {}},
                     3: {2: {}}}

        self.p4_edgelist = [(0, 1), (1, 2), (2, 3)]


    def test_is_planar_edgelist_input(self):
        P = planarity.PGraph(self.p4_edgelist)
        assert P.is_planar() is True
        P = planarity.PGraph(self.k5_edgelist)
        assert P.is_planar() is False

    def test_is_planar_edgelist_input_function(self):
        assert planarity.is_planar(self.k5_edgelist) is False

    def test_is_planar_adj_input(self):
        P = planarity.PGraph(self.p4_adj)
        assert P.is_planar() is True
        P=planarity.PGraph(self.k5_adj)
        assert P.is_planar() is False

    def test_is_planar_adj_input_function(self):
        assert planarity.is_planar(self.k5_adj) is False

    def test_is_planar_adj_symmetric(self):
        P = planarity.PGraph(self.k5_adj_symmetric)
        assert P.is_planar() is False

    def test_is_planar_adj_set(self):
        P = planarity.PGraph(self.k5_adj_set)
        assert P.is_planar() is False

    def test_is_planar_adj_list(self):
        P = planarity.PGraph(self.k5_adj_list)
        assert P.is_planar() is False

    def test_goldner_harary(self):
        # goldner-harary graph
        # http://en.wikipedia.org/wiki/Goldner%E2%80%93Harary_graph
        # a maximal planar graph
        e = [(1,2), ( 1,3 ),( 1,4 ),( 1,5 ),( 1,7 ),( 1,8 ),( 1,10 ),
            (1,11 ),( 2,3 ),( 2,4 ),( 2,6 ),( 2,7 ),( 2,9 ),( 2,10 ),
            ( 2,11 ),( 3,4 ),( 4,5 ),( 4,6 ),( 4,7 ),( 5,7 ),( 6,7 ),
            ( 7,8 ),( 7,9 ),( 7,10 ),( 8,10 ),( 9,10 ),( 10,11)]
        P = planarity.PGraph(e)
        assert P.is_planar() is True

    def test_kuratowski_k5(self):
        P = planarity.PGraph(self.k5_edgelist)
        edges = P.kuratowski_edges()
        assert frozenset(frozenset(x) for x in edges) == frozenset(frozenset(x) for x in self.k5_edgelist)

    def test_kuratowski_k5_function(self):
        edges = planarity.kuratowski_edges(self.k5_edgelist)
        assert frozenset(frozenset(x) for x in edges) == frozenset(frozenset(x) for x in self.k5_edgelist)

    def test_no_kuratowski_k5m(self):
        edges = self.k5_edgelist[:]
        edges.remove((0,1))
        P = planarity.PGraph(edges)
        edges = P.kuratowski_edges()
        assert frozenset(edges) == frozenset()

    def test_draw_text(self):
        e = ([1,2],)
        P = planarity.PGraph(e)
        s = P.ascii()#.decode()
        assert s == '1\n|\n2\n \n'

    def test_write_adjlist(self):
        e = ([1,2],)
        P = planarity.PGraph(e)
        fname = tempfile.mktemp()
        P.write(fname)
        d = open(fname).read()
        answer = 'N=2\n1: 2 0\n2: 1 0\n'
        assert d == answer
        os.unlink(fname)



import planarity
# Example of the complete graph of 5 nodes, K5
# K5 is not planar

# use text strings as labels
edgelist = [('a', 'b'), ('a', 'c'), ('a', 'd'), ('a', 'e'),
            ('b', 'c'),('b', 'd'),('b', 'e'),
            ('c', 'd'), ('c', 'e'),
            ('d', 'e')]

# remove an edge
edgelist.remove(('a','b'))
# graph is now planar
# make text drawing
print(planarity.ascii(edgelist))



import planarity
# Example of the complete graph of 5 nodes, K5
# K5 is not planar
# any of the following formats can bed used for representing the graph

edgelist = [(0, 1), (0, 2), (0, 3), (0, 4),
            (1, 2),(1, 3),(1, 4),
            (2, 3), (2, 4),
            (3, 4)]
P=planarity.PGraph(edgelist)
print(P.nodes()) # indexed from 1..n
print(P.mapping()) # the node mapping
print(P.edges()) # edges
print(P.is_planar())  # False
print(P.kuratowski_edges())

edgelist.remove((0,1))
P=planarity.PGraph(edgelist)
print(P.ascii())



"""Functional interface to planarity."""
import planarity

__all__ = ['is_planar', 'kuratowski_edges', 'ascii', 'write', 'mapping']

def is_planar(graph):
    """Test planarity of graph."""
    return planarity.PGraph(graph).is_planar()

def kuratowski_edges(graph):
    """Return edges of forbidden subgraph of non-planar graph."""
    return planarity.PGraph(graph).kuratowski_edges()

def ascii(graph):
    """Draw text representation of a planar graph."""
    return planarity.PGraph(graph).ascii()

def write(graph, path='stdout'):
    """Write an adjacency list representation of graph to path."""
    planarity.PGraph(graph).write(path)

def mapping(graph):
    """Return dictionary of internal mapping of nodes to integers."""
    return planarity.PGraph(graph).mapping()



#!python
#cython: embedsignature=True
"""
Wrapper for Boyer's (C) planarity algorithms.
"""
from planarity cimport cplanarity
from libc.stdlib cimport free
import warnings

cdef class PGraph:
    cdef cplanarity.graphP theGraph
    cdef dict nodemap
    cdef dict reverse_nodemap
    cdef int embedding 
    def __init__(self,graph):
        # guess input type
        if hasattr(graph,'nodes'):
            # NetworkX graph
            nodes=list(graph.nodes())
            edges=list(graph.edges())
        elif hasattr(graph,'keys'):
            # adjacency dict of dicts|sets|lists
            nodes=graph.keys()
            edges=[]
            seen=set()
            for node,adj in graph.items():
                nbrs=[n for n in adj if n not in seen]
                l=len(nbrs)
                edges.extend(zip([node]*l,nbrs))
                seen.add(node)
        else:
            # edge list (list of lists|tuples)
            try:
                nodes=set([node for sublist in graph for node in sublist])
            except:
                raise RuntimeError("Unknown input type")
            edges=graph
        n=len(nodes)
        self.nodemap=dict(zip(nodes,range(1,n+1)))
        self.reverse_nodemap=dict(zip(range(1,n+1),nodes))
        self.theGraph = cplanarity.gp_New()
        cdef int status
        status = cplanarity.gp_InitGraph(self.theGraph, n)
        if status != cplanarity.OK:
            raise RuntimeError("planarity: failed to initialize graph")
        # add the edges and check return
        seen = set()
        for u,v in edges:
            if (u,v) not in seen and (v,u) not in seen:
                status = cplanarity.gp_AddEdge(self.theGraph, 
                                               self.nodemap[u], 0, 
                                               self.nodemap[v], 0)
                if status == cplanarity.NOTOK:
                    cplanarity.gp_Free(&self.theGraph)
                    raise RuntimeError("planarity: failed adding edge.")
                seen.add((u,v))
            else:
                warnings.warn('ignoring parallel edge %s-%s'%(str(u),str(v)))
        self.embedding=cplanarity.NOTOK


    def __dealloc__(self):
        cplanarity.gp_Free(&self.theGraph)


    def embed_planar(self):
        if self.embedding == 0:
            self.embedding = cplanarity.gp_Embed(self.theGraph, 
                                             cplanarity.EMBEDFLAGS_PLANAR)
            cplanarity.gp_SortVertices(self.theGraph)                  


    def embed_drawplanar(self):
        status = cplanarity.gp_AttachDrawPlanar(self.theGraph)
        if status == cplanarity.NOTOK:
            raise RuntimeError("planarity: failed attaching drawplanar.")
        status = cplanarity.gp_Embed(self.theGraph, 
                                             cplanarity.EMBEDFLAGS_DRAWPLANAR)
        if status == cplanarity.NONEMBEDDABLE:
            raise RuntimeError("planarity: graph not planar.")
        cplanarity.gp_SortVertices(self.theGraph)                  


    def is_planar(self):
        """Return True if graph is planar."""
        self.embed_planar()
        if  self.embedding == cplanarity.NONEMBEDDABLE:
            return False
        return True


    def kuratowski_edges(self):
        if self.is_planar():
            return []
        elif self.embedding == cplanarity.NONEMBEDDABLE:
            return self.edges(data=False)
        else:
            raise RuntimeError("planarity: Unknown error.")        


    def nodes(self,data=False):
        DRAWPLANAR_ID=1
        cdef cplanarity.DrawPlanarContext *context 
        drawing=cplanarity.gp_FindExtension(self.theGraph, 
                                            DRAWPLANAR_ID, 
                                            <void *> &context)        

        first=cplanarity.gp_GetFirstVertex(self.theGraph)
        last=cplanarity.gp_GetLastVertex(self.theGraph)+1
        r=self.reverse_nodemap
        nodes=[]
        for n in range(first,last):
            if data:
                data={}
                if drawing==1:
                    data.update(pos=context.VI[n].pos,
                                start=context.VI[n].start,
                                end=context.VI[n].end)
                nodes.append((r[n],data))
            else:
                nodes.append((r[n]))
        return nodes


    def edges(self,data=False):
        DRAWPLANAR_ID=1
        cdef cplanarity.DrawPlanarContext *context 
        drawing=cplanarity.gp_FindExtension(self.theGraph, 
                                            DRAWPLANAR_ID, 
                                            <void *> &context)        
        edges=[]
        r=self.reverse_nodemap
        first=cplanarity.gp_GetFirstVertex(self.theGraph)
        last=cplanarity.gp_GetLastVertex(self.theGraph)+1
        for n in range(first,last):
            e=cplanarity.gp_GetFirstArc(self.theGraph,n)
            isarc=cplanarity.gp_IsArc(e)
            while isarc > 0:
                nbr=cplanarity.gp_GetNeighbor(self.theGraph,e)
                if nbr > n:
                    if data:
                        data={}
                        if drawing==1:
                            data.update(pos=context.E[e].pos,
                                        start=context.E[e].start,
                                        end=context.E[e].end)
                        edges.append((r[n],r[nbr],data))
                    else:
                        edges.append((r[n],r[nbr]))
                e=cplanarity.gp_GetNextArc(self.theGraph,e)
                isarc=cplanarity.gp_IsArc(e)
        return edges


    def ascii(self):
        cdef char* s = NULL
        self.embed_drawplanar()
        status = cplanarity.gp_DrawPlanar_RenderToString(self.theGraph, &s)
        py_bytes = s[:]
        free(s)
        return py_bytes.decode('ascii')


    def write(self,path):
        bpath=path.encode()
        status=cplanarity.gp_Write(self.theGraph, bpath, 
                                   cplanarity.WRITE_ADJLIST)    
        
    def mapping(self):
        return self.reverse_nodemap
